@@ -167,6 +167,152 @@ void DiagramSVG::DrawAxes (const ptime& x_min, const ptime& x_max, XUnit x_unit,
 	}
 }
 
+	static string fold (const string& timestamp, Diagram::FoldInterval fi)
+	{
+		string result(timestamp);
+		switch (fi)
+		{
+			case Diagram::year:	result.erase(0,  5); result.erase(2); break;
+			case Diagram::month:	result.erase(0,  8); result.erase(2); break;
+			case Diagram::day:	result.erase(0, 11); result.erase(2); break;
+			case Diagram::hour:	result.erase(0, 14); result.erase(2); break;
+			case Diagram::minute:	result.erase(0, 17); break;
+		}
+		return result;
+	}
+
+void DiagramSVG::DrawAxes (Diagram::FoldInterval fi, double y_min, double y_max, double y_unit, RGB color)
+{
+	// for folding we must handle dates in the year 9999, this is a bit
+	// special but indeed no problem, we just need an according axis
+	switch (fi)
+	{
+		case year:
+		{
+			m_xmin = ptime(date(9999,  Jan, 1), time_duration(0, 0, 0));
+			m_xmax = ptime(date(10000, Jan, 1), time_duration(0, 0, 0));
+			m_xunit = months;
+			break;
+		}
+		case month:
+		{
+			m_xmin = ptime(date(9999, Jan,  1), time_duration(0, 0, 0));
+			m_xmax = ptime(date(9999, Feb,  1), time_duration(0, 0, 0));
+			m_xunit = days;
+			break;
+		}
+		case day:
+		{
+			m_xmin = ptime(date(9999, Jan,  1), time_duration(0, 0, 0));
+			m_xmax = ptime(date(9999, Jan,  2), time_duration(0, 0, 0));
+			m_xunit = hours;
+			break;
+		}
+		case hour:
+		{
+			m_xmin = ptime(date(9999, Jan,  1), time_duration(0, 0, 0));
+			m_xmax = ptime(date(9999, Jan,  1), time_duration(1, 0, 0));
+			m_xunit = minutes;
+			break;
+		}
+		case minute:
+		{
+			m_xmin = ptime(date(9999, Jan,  1), time_duration(0, 0, 0));
+			m_xmax = ptime(date(9999, Jan,  1), time_duration(0, 1, 0));
+			m_xunit = seconds;
+			break;
+		}
+	}
+
+	m_ymin = y_min;
+	m_ymax = y_max;
+	m_yunit = y_unit;
+
+	// approximate character dimensions
+	//
+	// Note: libboard doesn't compute real dimensions for text, if we
+	// put text on a shape and ask the shape about the size of the text
+	// it returns (0,0), so we must guess the dimensions, mysterious:
+	// it seems that we get good results if width and heightare both
+	// 5.0 units
+	//
+	const double char_width = 5.0, char_height = 5.0;
+
+	// Step 1: x-axis
+	time_duration increment(not_a_date_time);
+	NormalizeX(m_xmin, increment);
+
+	const double x_0 = CalcX(m_xmin);
+	m_shapes << Line(x_0, 0.0, CalcX(m_xmax + increment), 0.0, COLOR(color), LINE_THICKNESS);
+
+	const double dash_height = m_height / 100.0; // one percent of m_height
+	if (m_xunit == months)
+	{
+		// the problem with months is that they differ in their
+		// lengths, so we must rather increment the month in the
+		// date of the first day than adding just a fixed length as
+		// in all the other cases below
+		for (ptime t(m_xmin); t <= m_xmax; )
+		{
+			double x = CalcX(t);
+			m_shapes << Line(x, 0.0, x, - dash_height, COLOR(color), LINE_THICKNESS);
+			const string& s = fold(lexical_cast<string, ptime>(t), fi);
+			Text text(x + (char_height / 2.0), - dash_height - (s.length() * char_width), s, Fonts::Helvetica, 8.0, COLOR(color));
+			text.rotate(1.5707963); // 90°
+			m_shapes << text;
+			const date& d = t.date();
+			greg_month m = d.month();
+			t = ptime((m == 12) ? date(d.year() + 1, 1, 1) : date(d.year(), m + 1, 1), time_duration(0, 0, 0, 0));
+		}
+	}
+	else
+	{
+		for (ptime t(m_xmin); t <= m_xmax; t += increment)
+		{
+			double x = CalcX(t);
+			m_shapes << Line(x, 0.0, x, - dash_height, COLOR(color), LINE_THICKNESS);
+			const string& s = fold(lexical_cast<string, ptime>(t), fi);
+			Text text(x + (char_height / 2.0), - dash_height - (s.length() * char_width), s, Fonts::Helvetica, 8.0, COLOR(color));
+			text.rotate(1.5707963); // 90°
+			m_shapes << text;
+		}
+	}
+
+	// Step 2: y-axis, much easier because of simple values
+	//
+	// we must draw the y-axis some pixels longer on the upper end to
+	// avoid the top most number to be clipped, the shape will not
+	// know about the fact that the top most number has a text height
+	// that extends the axis
+	//
+	m_shapes << Line(x_0, CalcY(m_ymin), x_0, CalcY(m_ymax) + 5.0, COLOR(color), LINE_THICKNESS);
+
+	const double dash_width = m_width / 100.0; // one percent of m_width
+
+	string yf; // format string for the numbers on the y-axis
+	{
+		stringstream ss;
+		ss << m_yunit;
+		string s(ss.str());
+		string::size_type pos = s.find('.');
+		if (pos != string::npos)
+		{
+			s.erase(0, pos + 1);
+			yf = (format("%%1.%df") % s.length()).str();
+		}
+		else
+			yf = "%1.0f";
+	}
+
+	for (double t = m_ymin; t <= m_ymax; t += m_yunit)
+	{
+		double y = CalcY(t);
+		m_shapes << Line(x_0 - dash_width, y, x_0, y, COLOR(color), LINE_THICKNESS);
+		string s((format(yf) % t).str());
+		m_shapes << Text(x_0 - dash_width - (s.length() * char_width), y - (char_height / 2.0), s, Fonts::Helvetica, 8.0, COLOR(color));
+	}
+}
+
 void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, double thickness, bool continuous, int bar1, int bar2)
 {
 	const size_t* pDimensions = s.shape(); // [0] rows, [1] columns
@@ -217,12 +363,12 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 						double x_width = (CalcX(timestamp_right) - x) / double(bar2);
 						x += x_width * double(bar1 - 1);
 						double y = CalcY(any_cast<double>(current.m_value));
-						m_shapes << Rectangle(x, y, x_width, y - CalcY(m_ymin), COLOR(color), COLOR(color), 0);
+						m_shapes << LibBoard::Rectangle(x, y, x_width, y - CalcY(m_ymin), COLOR(color), COLOR(color), 0);
 					}
 					else
 					{
 						double x = CalcX(current.m_timestamp);
-						m_shapes << Line(x, CalcY(m_ymin), x, CalcY(any_cast<double>(current.m_value)), COLOR(color), thickness);
+						m_shapes << Line(x, CalcY(m_ymin), x, CalcY(any_cast<double>(current.m_value)), COLOR(color), (float)thickness);
 					}
 				}
 				break;
@@ -234,8 +380,8 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 					double x = CalcX(current.m_timestamp);
 					double y = CalcY(any_cast<double>(current.m_value));
 					const double extent = m_width / 100.0; // one percent of m_width
-					m_shapes << Line(x - extent, y, x + extent, y, COLOR(color), thickness);
-					m_shapes << Line(x, y - extent, x, y + extent, COLOR(color), thickness);
+					m_shapes << Line(x - extent, y, x + extent, y, COLOR(color), (float)thickness);
+					m_shapes << Line(x, y - extent, x, y + extent, COLOR(color), (float)thickness);
 				}
 				break;
 			}
@@ -246,7 +392,7 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 					double x = CalcX(current.m_timestamp);
 					double y = CalcY(any_cast<double>(current.m_value));
 					const double extent = m_width / 100.0; // one percent of m_width
-					m_shapes << Line(x, y - extent, x, y + extent, COLOR(color), thickness);
+					m_shapes << Line(x, y - extent, x, y + extent, COLOR(color), (float)thickness);
 				}
 				break;
 			}
@@ -257,7 +403,7 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 					double x = CalcX(current.m_timestamp);
 					double y = CalcY(any_cast<double>(current.m_value));
 					const double extent = m_width / 100.0; // one percent of m_width
-					m_shapes << Line(x - extent, y, x + extent, y, COLOR(color), thickness);
+					m_shapes << Line(x - extent, y, x + extent, y, COLOR(color), (float)thickness);
 				}
 				break;
 			}
@@ -268,8 +414,8 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 					double x = CalcX(current.m_timestamp);
 					double y = CalcY(any_cast<double>(current.m_value));
 					const double extent = m_width / 100.0; // one percent of m_width
-					m_shapes << Line(x - extent, y - extent, x + extent, y + extent, COLOR(color), thickness);
-					m_shapes << Line(x - extent, y + extent, x + extent, y - extent, COLOR(color), thickness);
+					m_shapes << Line(x - extent, y - extent, x + extent, y + extent, COLOR(color), (float)thickness);
+					m_shapes << Line(x - extent, y + extent, x + extent, y - extent, COLOR(color), (float)thickness);
 				}
 				break;
 			}
@@ -280,7 +426,7 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 					double x = CalcX(current.m_timestamp);
 					double y = CalcY(any_cast<double>(current.m_value));
 					const double extent = m_width / 100.0; // one percent of m_width
-					m_shapes << Circle(x, y, extent, COLOR(color), Color(false), thickness);
+					m_shapes << Circle(x, y, extent, COLOR(color), Color(false), (float)thickness);
 				}
 				break;
 			}
@@ -291,7 +437,7 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 					m_shapes << Line(
 						CalcX(last.m_timestamp), CalcY(any_cast<double>(last.m_value)),
 						CalcX(current.m_timestamp), CalcY(any_cast<double>(current.m_value)),
-						COLOR(color), thickness
+						COLOR(color), (float)thickness
 					);
 				}
 				break;
@@ -306,7 +452,7 @@ void DiagramSVG::DrawCurve (const Selection& s, RGB color, LineStyle style, doub
 void DiagramSVG::DrawVLine (const ptime& timestamp, RGB color, double thickness)
 {
 	double x = CalcX(timestamp);
-	m_shapes << Line(x, CalcY(m_ymin), x, CalcY(m_ymax), COLOR(color), (thickness == 0.0) ? LINE_THICKNESS : thickness);
+	m_shapes << Line(x, CalcY(m_ymin), x, CalcY(m_ymax), COLOR(color), (float)((thickness == 0.0) ? LINE_THICKNESS : thickness));
 }
 
 void DiagramSVG::DrawHLine (double value, RGB color, double thickness)
@@ -320,7 +466,7 @@ void DiagramSVG::DrawHLine (double value, RGB color, double thickness)
 	time_duration increment(not_a_date_time);
 	NormalizeX(dummy, increment);
 
-	m_shapes << Line(CalcX(m_xmin), y, CalcX(m_xmax + increment), y, COLOR(color), (thickness == 0.0) ? LINE_THICKNESS : thickness);
+	m_shapes << Line(CalcX(m_xmin), y, CalcX(m_xmax + increment), y, COLOR(color), (float)((thickness == 0.0) ? LINE_THICKNESS : thickness));
 }
 
 void DiagramSVG::SaveAs (const string& filename) const
@@ -346,7 +492,7 @@ double DiagramSVG::CalcX (const ptime& x) const
 double DiagramSVG::CalcY (double y) const
 {
 	double a = y - m_ymin;
-	double_t b = m_ymax - m_ymin;
+	double b = m_ymax - m_ymin;
 	return (a / b) * m_height;
 }
 
